package com.xydl.cac.controller;

import com.xydl.cac.entity.Bdz;
import com.xydl.cac.model.Response;
import com.xydl.cac.service.BdzService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.List;

@RestController
@Api(tags = {"变电站相关接口"})
@RequestMapping("bdz")
@Slf4j
public class BdzController extends BasicController {

    @Resource
    BdzService service;

    @GetMapping("listAll")
    @ApiOperation("查询全部列表")
    public Response<List<Bdz>> listAll() {
        try {
            List<Bdz> result = service.listAll();
            return Response.success(result);
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @PostMapping("add")
    @ApiOperation("新增")
    public Response<Bdz> add(@Validated @RequestBody Bdz item) {
        try {
            Bdz result = service.add(item);
            return Response.success(result);
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @PostMapping("update")
    @ApiOperation("更新")
    public Response<String> update(@Validated @RequestBody Bdz item) {
        try {
            if (item.getId() == null) {
                throw new Exception("ID不能为空!");
            }
            service.update(item);
            return Response.success("OK");
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @PostMapping("delete")
    @ApiOperation("删除")
    public Response<String> delete(@Validated @NotNull(message = "ID不能为空!") Integer id) {
        try {
            service.delete(id);
            return Response.success("OK");
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

}
