package com.xydl.cac.controller;

import com.xydl.cac.entity.IcdFileConfig;
import com.xydl.cac.model.Response;
import com.xydl.cac.service.DataService;
import com.xydl.cac.service.IcdFileConfigService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.nio.charset.Charset;
import java.util.List;

@RestController
@Api(tags = {"IcdConfig相关接口"})
@RequestMapping("icdconfig")
@Slf4j
public class IcdConfigController extends BasicController {

    @Resource
    IcdFileConfigService configService;
    @Resource
    DataService dataService;

    @PostMapping("upload")
    @ApiOperation("处理icd文件")
    public Response<String> upload(@RequestParam("file") MultipartFile file) throws Exception {
        if (file != null && file.getInputStream() != null) {
            String content = IOUtils.toString(file.getInputStream(), Charset.forName("UTF-8"));
            configService.upload(content);
            return Response.success("OK");
        } else {
            return Response.fail("缺少上传文件");
        }
    }

    @GetMapping("list")
    @ApiOperation("查询全部类型列表")
    public Response<List<IcdFileConfig>> list(String iedName) {
        try {
            List<IcdFileConfig> result = configService.list(iedName);
            return Response.success(result);
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @GetMapping("iedList")
    @ApiOperation("查询IED列表")
    public Response<List<String>> iedList() {
        try {
            List<String> result = configService.iedList();
            return Response.success(result);
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @PostMapping("update")
    @ApiOperation("更新ICD类型配置")
    public Response<String> update(@RequestBody IcdFileConfig item) {
        try {
            configService.update(item);
            return Response.success("OK");
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @PostMapping("delete")
    @ApiOperation("删除ICD类型配置")
    public Response<String> delete(@Validated @NotNull(message = "ID不能为空!") Integer id) {
        try {
            configService.delete(id);
            return Response.success("OK");
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @GetMapping("tableList")
    @ApiOperation("查询data表名")
    public Response<List<String>> tableList() {
        try {
            List<String> result = dataService.getDataTables();
            return Response.success(result);
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @GetMapping("colList")
    @ApiOperation("查询data表字段名")
    public Response<List<String>> colList(String tableName) {
        try {
            List<String> result = dataService.getDataTableColumns(tableName);
            return Response.success(result);
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

}
