package com.xydl.cac.controller;

import com.xydl.cac.entity.ModevType;
import com.xydl.cac.model.Response;
import com.xydl.cac.service.ModevTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.List;

@RestController
@Api(tags = {"监测设备类型相关接口"})
@RequestMapping("modevtype")
@Slf4j
public class ModevTypeController extends BasicController {

    @Resource
    ModevTypeService service;

    @GetMapping("listAll")
    @ApiOperation("查询列表")
    public Response<List<ModevType>> listAll(Integer bdzid) {
        try {
            List<ModevType> result = service.listAll(bdzid);
            return Response.success(result);
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @PostMapping("add")
    @ApiOperation("新增")
    public Response<ModevType> add(@Validated @RequestBody ModevType item) {
        try {
            ModevType result = service.add(item);
            return Response.success(result);
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @PostMapping("update")
    @ApiOperation("更新")
    public Response<String> update(@Validated @RequestBody ModevType item) {
        try {
            if (item.getId() == null) {
                throw new Exception("ID不能为空!");
            }
            service.update(item);
            return Response.success("OK");
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

    @PostMapping("delete")
    @ApiOperation("删除")
    public Response<String> delete(@Validated @NotNull(message = "ID不能为空!")  Integer id) {
        try {
            service.delete(id);
            return Response.success("OK");
        } catch (Exception ex) {
            return Response.fail(ex.getMessage());
        }
    }

}
