package com.xydl.cac.service.impl;

import com.xydl.cac.entity.NiecPoint;
import com.xydl.cac.entity.NiecSensor;
import com.xydl.cac.model.ConditionModel;
import com.xydl.cac.model.OnePage;
import com.xydl.cac.service.DataService;
import com.xydl.cac.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class DataServiceImpl implements DataService {

    @Resource
    private JdbcTemplate jdbcTemplate;

    @Override
    public List<String> getDataTables() throws Exception {
        List<String> tables = new ArrayList<>();
        String sql = "SHOW TABLES";
        List<String> list = jdbcTemplate.queryForList(sql, String.class);
        for (String table : list) {
            if (table.startsWith("data_")) {
                tables.add(table);
            }
        }
        return tables;
    }

    @Override
    public List<String> getDataTableColumns(String tableName) throws Exception {
        List<String> cols = new ArrayList<>();
        String sql = "SELECT COLUMN_NAME FROM information_schema.columns WHERE TABLE_NAME='" + tableName + "'";
        List<String> list = jdbcTemplate.queryForList(sql, String.class);
        for (String col : list) {
            if (!col.equals("id") && !col.equals("eqmid")
                    && !col.equals("acquisitionTime") && !col.equals("d_time")
                    && !col.equals("create_time") && !col.equals("update_time")
                    && !col.equals("isupload")) {
                cols.add(col);
            }
        }
        return cols;
    }

    @Override
    public OnePage<Map<String, Object>> getDate(NiecSensor sensor, List<NiecPoint> points, ConditionModel model) {
        OnePage result = new OnePage();

        String sqlFrom = " FROM " + sensor.getTableName();
        String sqlWhere = " WHERE " + sensor.getDevidFieldName() + "=" + sensor.getDevId();
        if (model.getStartTime() != null) {
            sqlWhere = sqlWhere + " AND " + sensor.getDtimeFieldName() + " >= '" + DateUtil.format(model.getStartTime()) + "'";
        }
        if (model.getEndTime() != null) {
            sqlWhere = sqlWhere + " AND " + sensor.getDtimeFieldName() + " < '" + DateUtil.format(model.getEndTime()) + "'";
        }
        String sqlOrder = " ORDER BY " + sensor.getDtimeFieldName() + " DESC";
        String sqlLimit = "";
        if (model.getPageNum() != null && model.getPageSize() != null) {
            result.setPageNum(model.getPageNum());
            result.setPageSize(model.getPageSize());
            sqlLimit = String.format(" LIMIT %d,%d", (model.getPageNum() - 1) * model.getPageSize(), model.getPageSize());
        }

        String sqlSelect = "SELECT DATE_FORMAT(" + sensor.getDtimeFieldName() + ", '%Y-%m-%d %H:%i:%s') as acquisitionTime";
        for (NiecPoint point : points) {
            if (StringUtils.isNotBlank(point.getField())) {
                sqlSelect = sqlSelect + ", " + point.getField();
            }
        }

        String sqlCount = "SELECT count(*)" + sqlFrom + sqlWhere;
        String sql = sqlSelect + sqlFrom + sqlWhere + sqlOrder + sqlLimit;
        log.info(sql);

        Long count = jdbcTemplate.queryForObject(sqlCount, Long.class);
        List<Map<String, Object>> content = jdbcTemplate.queryForList(sql);

        result.setTotal(count);
        result.setContent(content);
        return result;
    }
}
