package com.xydl.cac.service.impl;

import com.xydl.cac.entity.IcdFileConfig;
import com.xydl.cac.entity.IcdFileConfigInst;
import com.xydl.cac.repository.IcdFileConfigInstRepository;
import com.xydl.cac.repository.IcdFileConfigRepository;
import com.xydl.cac.service.IcdFileConfigService;
import com.xydl.cac.util.IcdXmlUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class IcdFileConfigServiceImpl implements IcdFileConfigService {

    @Resource
    IcdFileConfigRepository repository;
    @Resource
    IcdFileConfigInstRepository instRepository;
    @Resource
    private JdbcTemplate jdbcTemplate;

    @Override
    public void upload(String xml) throws Exception {
        LinkedHashMap<String, IcdFileConfig> result = IcdXmlUtil.loadIcdType(xml);
        Collection<IcdFileConfig> configList = result.values();
        for (IcdFileConfig config : configList) {
            List<IcdFileConfig> list = repository.findByIedNameAndLdeviceInstAndLnClassAndDoName(
                    config.getIedName(), config.getLdeviceInst(), config.getLnClass(), config.getDoName());
            if (CollectionUtils.isEmpty(list)) {
                repository.save(config);
                List<IcdFileConfigInst> instList = config.getInstList();
                for (IcdFileConfigInst item : instList) {
                    item.setConfigId(config.getId());
                }
                instRepository.saveAll(instList);
            }
        }
    }

    @Override
    public List<String> iedList() {
        String sql = "SELECT DISTINCT ied_name FROM icd_file_config";
        List<String> list = jdbcTemplate.queryForList(sql, String.class);
        return list;
    }

    @Override
    public List<IcdFileConfig> list(String iedName) throws Exception {
        List<IcdFileConfig> result;
        if (StringUtils.isNotBlank(iedName)) {
            result = repository.findByIedName(iedName);
        } else {
            result = repository.findAll();
        }
        for (IcdFileConfig item : result) {
            List<IcdFileConfigInst> list = instRepository.findByConfigId(item.getId());
            item.setInstList(list);
        }
        return result;
    }

    @Override
    public void update(IcdFileConfig item) {
        repository.save(item);
    }

    @Override
    public void delete(Integer id) {
        instRepository.deleteByConfigId(id);
        repository.deleteById(id);
    }

}
