package com.xydl.cac.service.impl;

import com.xydl.cac.entity.Bdz;
import com.xydl.cac.entity.Jg;
import com.xydl.cac.entity.Zsb;
import com.xydl.cac.repository.BdzRepository;
import com.xydl.cac.repository.JgRepository;
import com.xydl.cac.repository.ZsbRepository;
import com.xydl.cac.service.JgService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class JgServiceImpl implements JgService {

    @Resource
    JgRepository repository;
    @Resource
    BdzRepository bdzRepository;
    @Resource
    ZsbRepository zsbRepository;

    @Override
    public List<Jg> listAll(Integer bdzid) {
        if (bdzid == null) {
            List<Jg> result = repository.findAll();
            this.fillBdzName(result);
            return result;
        } else {
            List<Jg> result = repository.findByBdzid(bdzid);
            Optional<Bdz> optional = bdzRepository.findById(bdzid);
            if (optional.isPresent() && !CollectionUtils.isEmpty(result)) {
                for (Jg jg : result) {
                    jg.setBdzName(optional.get().getMc());
                }
            }
            return result;
        }
    }

    private void fillBdzName(List<Jg> list) {
        List<Bdz> bdzList = bdzRepository.findAll();
        if (!CollectionUtils.isEmpty(list) && !CollectionUtils.isEmpty(bdzList)) {
            for (Jg jg : list) {
                for (Bdz bdz : bdzList) {
                    if (bdz.getId().equals(jg.getBdzid())) {
                        jg.setBdzName(bdz.getMc());
                        break;
                    }
                }
            }
        }
    }

    @Override
    public Jg add(Jg item) throws Exception {
        item.setId(null);
        Optional<Bdz> optionalBdz = bdzRepository.findById(item.getBdzid());
        if (!optionalBdz.isPresent()) {
            throw new Exception("未找到该变电站");
        }
        return repository.save(item);
    }

    @Override
    public void update(Jg item) throws Exception {
        Optional<Bdz> optionalBdz = bdzRepository.findById(item.getBdzid());
        if (!optionalBdz.isPresent()) {
            throw new Exception("未找到该变电站");
        }
        repository.save(item);
    }

    @Override
    public void delete(Integer id) throws Exception {
        List<Zsb> zsbList = zsbRepository.findByJgid(id);
        if (!CollectionUtils.isEmpty(zsbList)) {
            throw new Exception("已被主设备使用不能删除");
        }
        repository.deleteById(id);
    }

    @Override
    public Jg detail(Integer id) throws Exception {
        Optional<Jg> optional = repository.findById(id);
        if (!optional.isPresent()) {
            throw new Exception("未找到该区域");
        }
        Jg jg = optional.get();
        Optional<Bdz> optionalBdz = bdzRepository.findById(jg.getBdzid());
        if (optionalBdz.isPresent()) {
            jg.setBdzName(optionalBdz.get().getMc());
        }
        return jg;
    }
}
