package com.xydl.cac.service.impl;

import com.xydl.cac.entity.NiecPoint;
import com.xydl.cac.entity.NiecSensor;
import com.xydl.cac.model.ConditionModel;
import com.xydl.cac.model.OnePage;
import com.xydl.cac.model.SensorDetail;
import com.xydl.cac.repository.NiecSensorRepository;
import com.xydl.cac.service.DataService;
import com.xydl.cac.service.NiecPointService;
import com.xydl.cac.service.NiecSensorService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class NiecSensorServiceImpl implements NiecSensorService {

    @Resource
    NiecSensorRepository repository;
    @Resource
    NiecPointService pointService;
    @Resource
    DataService dataService;

    @Override
    public List<NiecSensor> listAll() {
        List<NiecSensor> list = repository.findAll();
        return list;
    }

    @Override
    public Page<NiecSensor> list(int pageNum, int pageSize) {
        PageRequest request = PageRequest.of(pageNum, pageSize);
        Page<NiecSensor> result = repository.findAll(request);
        return result;
    }

    @Override
    public List<NiecSensor> getTree() {
        List<NiecSensor> list = this.listAll();
        List<NiecSensor> result = new ArrayList<>();
        for (NiecSensor item : list) {
            boolean done = false;
            if (item.getParentId() != null) {
                for (NiecSensor parent : list) {
                    if (parent.getId().intValue() == item.getParentId().intValue()) {
                        if (StringUtils.isNotBlank(item.getTableName())) {
                            parent.addChild(item);
                        }
                        done = true;
                        break;
                    }
                }
            }
            if (!done) {
                result.add(item);
            }
        }
        return result;
    }

    @Override
    public SensorDetail<Map<String, Object>> getDetail(ConditionModel model) throws Exception {
        Optional<NiecSensor> optional = repository.findById(model.getId());
        if (!optional.isPresent()) {
            throw new Exception("未找到该装置");
        }
        NiecSensor sensor = optional.get();
        if (StringUtils.isBlank(sensor.getDtimeFieldName())) {
            throw new Exception("该装置缺少采样时间字段名信息");
        }
        if (StringUtils.isBlank(sensor.getTableName())) {
            throw new Exception("该装置缺少table_name信息");
        }
        if (StringUtils.isBlank(sensor.getDevidFieldName())) {
            throw new Exception("该装置缺少sensor_id的字段名信息");
        }
        if (sensor.getDevId() == null) {
            throw new Exception("该装置缺少dev_id信息");
        }
        List<NiecPoint> points = pointService.getList(sensor.getId(), 2);
        if (CollectionUtils.isEmpty(points)) {
            throw new Exception("该装置缺少业务点表配置信息");
        }

        SensorDetail<Map<String, Object>> result = this.getData(sensor, points, model);
        return result;
    }

    public SensorDetail<Map<String, Object>> getData(NiecSensor sensor, List<NiecPoint> points, ConditionModel model) {
        List<NiecPoint> pointList = new ArrayList<>();
        for (NiecPoint point : points) {
            if (StringUtils.isNotBlank(point.getField())) {
                pointList.add(point);
            }
        }

        SensorDetail<Map<String, Object>> result = new SensorDetail();
        result.setSensor(sensor);
        OnePage<Map<String, Object>> page = dataService.getDate(sensor, pointList, model);
        result.initFrom(page);

        for (NiecPoint point : pointList) {
            List<Object> dataList = new ArrayList<>();
            for (Map<String, Object> item : result.getContent()) {
                Object value = item.get(point.getField());
                dataList.add(value);
            }
            point.setData(dataList);
        }
        result.setPoints(pointList);
        return result;
    }
}
