package com.xydl.cac.service.impl;

import com.xydl.cac.entity.Jg;
import com.xydl.cac.entity.Lx;
import com.xydl.cac.entity.Zsb;
import com.xydl.cac.repository.LxRepository;
import com.xydl.cac.repository.ZsbRepository;
import com.xydl.cac.service.JgService;
import com.xydl.cac.service.ZsbService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class ZsbServiceImpl implements ZsbService {

    @Resource
    ZsbRepository repository;
    @Resource
    JgService jgService;
    @Resource
    LxRepository lxRepository;

    @Override
    public List<Zsb> listAll(Integer jgid) throws Exception {
        if (jgid == null) {
            List<Zsb> result = repository.findAll();
            this.fillJgBdzName(result);
            this.fillLxName(result);
            return result;
        } else {
            List<Zsb> result = repository.findByJgid(jgid);
            Jg jg = jgService.detail(jgid);
            if (!CollectionUtils.isEmpty(result)) {
                for (Zsb zsb : result) {
                    zsb.setJgName(jg.getMc());
                    zsb.setBdzName(jg.getBdzName());
                }
                this.fillLxName(result);
            }
            return result;
        }
    }

    private void fillJgBdzName(List<Zsb> list) {
        List<Jg> jgList = jgService.listAll(null);
        if (!CollectionUtils.isEmpty(list) && !CollectionUtils.isEmpty(jgList)) {
            for (Zsb zsb : list) {
                for (Jg jg : jgList) {
                    if (jg.getId().equals(zsb.getJgid())) {
                        zsb.setBdzName(jg.getBdzName());
                        zsb.setJgName(jg.getMc());
                        break;
                    }
                }
            }
        }
    }

    private void fillLxName(List<Zsb> list) {
        List<Lx> lxList = lxRepository.findAll();
        if (!CollectionUtils.isEmpty(list) && !CollectionUtils.isEmpty(lxList)) {
            for (Zsb zsb : list) {
                for (Lx lx : lxList) {
                    if (lx.getId().equals(zsb.getLxid())) {
                        zsb.setLxName(lx.getMc());
                        break;
                    }
                }
            }
        }
    }

    @Override
    public Zsb add(Zsb item) throws Exception {
        item.setId(null);
        Jg jg = jgService.detail(item.getJgid());
        item.setBdzid(jg.getBdzid());
        Optional<Lx> optionalLx = lxRepository.findById(item.getLxid());
        if (!optionalLx.isPresent()) {
            throw new Exception("未找到该设备类型");
        }
        return repository.save(item);
    }

    @Override
    public void update(Zsb item) throws Exception {
        Jg jg = jgService.detail(item.getJgid());
        item.setBdzid(jg.getBdzid());
        Optional<Lx> optionalLx = lxRepository.findById(item.getLxid());
        if (!optionalLx.isPresent()) {
            throw new Exception("未找到该设备类型");
        }
        repository.save(item);
    }

    @Override
    public void delete(Integer id) throws Exception {
        repository.deleteById(id);
    }

    @Override
    public Zsb detail(Integer id) throws Exception {
        Optional<Zsb> optional = repository.findById(id);
        if (!optional.isPresent()) {
            throw new Exception("未找到该主设备");
        }
        Zsb zsb = optional.get();
        Jg jg = jgService.detail(zsb.getJgid());
        zsb.setBdzName(jg.getBdzName());
        zsb.setJgName(jg.getMc());
        Optional<Lx> optionalLx = lxRepository.findById(zsb.getLxid());
        if (optionalLx.isPresent()) {
            zsb.setLxName(optionalLx.get().getMc());
        }
        return zsb;
    }
}
